# Copyright (C) 2021 Bosutech XXI S.L.
#
# nucliadb is offered under the AGPL v3.0 and as commercial software.
# For commercial licensing, contact us at info@nuclia.com.
#
# AGPL:
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

from dataclasses import dataclass
from typing import Optional

from pydantic import BaseModel


class ObjectInfo(BaseModel):
    name: str


class ObjectMetadata(BaseModel):
    filename: str
    content_type: str
    size: int


@dataclass
class Range:
    """
    Represents a range of bytes to be downloaded from a file. The range is inclusive.
    The start and end values are 0-based.
    """

    start: Optional[int] = None
    end: Optional[int] = None

    def any(self) -> bool:
        return self.start is not None or self.end is not None

    def to_header(self) -> str:
        return f"bytes={self.start or 0}-{self.end or ''}"
